<?php

namespace Drupal\llms_txt_generator\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configuration form for the LLMs.txt Generator module.
 */
class LlmsTextGeneratorSettingsForm extends ConfigFormBase {

  /**
   * The file generator service.
   *
   * @var \Drupal\llms_txt_generator\Service\LlmsFileGenerator
   */
  protected $fileGenerator;

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->fileGenerator = $container->get('llms_txt_generator.file_generator');
    $instance->requestStack = $container->get('request_stack');
    return $instance;
  }

  /**
   * {@inheritDoc}
   */
  public function getFormId() {
    return 'llms_txt_generator_settings_form';
  }

  /**
   * {@inheritDoc}
   */
  protected function getEditableConfigNames() {
    return ['llms_txt_generator.settings'];
  }

  /**
   * {@inheritDoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('llms_txt_generator.settings');

    $form['info'] = [
      '#type' => 'markup',
      '#markup' => '<p>' . $this->t('The llms.txt file helps control how Large Language Models (LLMs) like ChatGPT, Claude, and others interact with your website content.') . '</p>',
    ];

    $form['llms_txt_content'] = [
      '#type' => 'textarea',
      '#title' => $this->t('LLMs.txt Content'),
      '#description' => $this->t('Enter the content for your llms.txt file. This will be accessible at /llms.txt. You can include sections with site information and URL listings as shown in the default example.'),
      '#default_value' => $config->get('llms_txt_content') ?: $this->getDefaultLlmsTxtContent(),
      '#rows' => 25,
      '#required' => TRUE,
    ];

    $form['reset_to_default'] = [
      '#type' => 'button',
      '#value' => $this->t('Reset to default content'),
      '#attributes' => [
        'onclick' => 'document.getElementById("edit-llms-txt-content").value = ' . json_encode($this->getDefaultLlmsTxtContent()) . '; return false;',
      ],
    ];

    $form['enable_file'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable llms.txt file'),
      '#description' => $this->t('Check this box to make the llms.txt file available. Uncheck to disable the file.'),
      '#default_value' => $config->get('enable_file') ?? TRUE,
    ];

    $form['reference'] = [
      '#type' => 'details',
      '#title' => $this->t('Section Examples'),
      '#open' => FALSE,
      '#collapsible' => TRUE,
    ];

    $form['reference']['content'] = [
      '#markup' => '
        <h3>' . $this->t('Example Sections for llms.txt') . '</h3>
        <p>' . $this->t('You can copy and paste these section examples into your llms.txt content:') . '</p>

        <h4>' . $this->t('Site Information') . '</h4>
        <pre>
**Site Type:** Website, Content Repository
**Purpose:** Information Sharing, Documentation
**Authors:** Your Team Name
        </pre>

        <h4>' . $this->t('Access Guidelines') . '</h4>
        <pre>
## Access Guidelines

- Base Rate: 100 requests per hour per IP
- Cache Retention: 24 hour maximum
- Content Usage: Attribution required
- Commercial Use: Requires written permission
- Training Usage: Permitted for public documentation only
        </pre>

        <h4>' . $this->t('URL Section Example') . '</h4>
        <pre>
## Documentation

- [Page Title](https://example.com/page): Page description goes here
- [Another Page](https://example.com/another): Another description
- [Resource Page](https://example.com/resource): Resource description
        </pre>

        <h4>' . $this->t('For Human Visitors') . '</h4>
        <pre>
## For Human Visitors

- Main Website: https://example.com
- Contact: webmaster@example.com
- About llms.txt: https://llmstxt.org

Version: 1.0 (Updated: ' . date('M d Y') . ')
        </pre>
      ',
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritDoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    // Validate llms.txt content.
    $llms_txt_content = $form_state->getValue('llms_txt_content');

    // Check if content is not just whitespace.
    if (trim($llms_txt_content) === '') {
      $form_state->setErrorByName('llms_txt_content', $this->t('The llms.txt content cannot be empty.'));
      return;
    }

    $lines = explode("\n", $llms_txt_content);
    $directive_lines = 0;
    $total_lines = count($lines);

    foreach ($lines as $line_number => $line) {
      $line = trim($line);
      // Skip empty lines and comments.
      if (empty($line) || strpos($line, '#') === 0) {
        continue;
      }

      // Count lines that appear to be directives.
      if (strpos($line, ':') !== FALSE) {
        $directive_lines++;
      }
    }

    // Only show a warning if no directive lines were found at all.
    if ($total_lines > 5 && $directive_lines === 0) {
      $this->messenger()
        ->addWarning($this->t('Your llms.txt does not appear to contain any directives (lines with colon separators). While this is allowed, it may not be effective for controlling LLM behavior.'));
    }
  }

  /**
   * {@inheritDoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('llms_txt_generator.settings')
      ->set('llms_txt_content', $form_state->getValue('llms_txt_content'))
      ->set('enable_file', $form_state->getValue('enable_file'))
      ->save();

    // Generate the file.
    $result = $this->fileGenerator->generateLlmsFile();

    if ($result) {
      $this->messenger()
        ->addMessage($this->t('The llms.txt file has been successfully generated.'));
    }
    else {
      $this->messenger()
        ->addError($this->t('The configuration was saved, but the llms.txt file could not be generated.'));
    }

    parent::submitForm($form, $form_state);
  }

  /**
   * Returns default content for the llms.txt file.
   *
   * @return string
   *   Default llms.txt content.
   */
  protected function getDefaultLlmsTxtContent() {
    return $this->fileGenerator->getDefaultTemplateContent();
  }

}
